@@ -1,11 +1,11 @@
-/* $Id: config.h,v 1.4 2011/05/23 12:22:29 nanard Exp $ */
+/* $Id: config.h,v 1.6 2012/09/27 15:40:29 nanard Exp $ */
 /*  MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
- * (c) 2006-2011 Thomas Bernard 
+ * (c) 2006-2011 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
-#ifndef __CONFIG_H__
-#define __CONFIG_H__
+#ifndef CONFIG_H_INCLUDED
+#define CONFIG_H_INCLUDED
 
 /* use BSD daemon() ? */
 #define USE_DAEMON
