@@ -1,12 +1,12 @@
-/* $Id: ifacewatch.h,v 1.1 2011/07/29 15:21:13 nanard Exp $ */
+/* $Id: ifacewatch.h,v 1.3 2012/09/27 15:40:29 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
- * (c) 2011 Thomas Bernard 
+ * (c) 2011 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
-#ifndef __IFACEWATCH_H__
-#define __IFACEWATCH_H__
+#ifndef IFACEWATCH_H_INCLUDED
+#define IFACEWATCH_H_INCLUDED
 
 int
 OpenAndConfInterfaceWatchSocket(void);
