@@ -1,6 +1,6 @@
-/* $Id: minissdpd.c,v 1.26 2011/10/07 09:45:19 nanard Exp $ */
+/* $Id: minissdpd.c,v 1.35 2012/05/21 17:13:11 nanard Exp $ */
 /* MiniUPnP project
- * (c) 2007-2011 Thomas Bernard
+ * (c) 2007-2012 Thomas Bernard
  * website : http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
@@ -61,6 +61,7 @@ struct device {
 
 #define NTS_SSDP_ALIVE	1
 #define NTS_SSDP_BYEBYE	2
+#define NTS_SSDP_UPDATE	3
 
 /* discovered device list kept in memory */
 struct device * devlist = 0;
@@ -69,11 +70,25 @@ struct device * devlist = 0;
 unsigned int upnp_bootid = 1;
 unsigned int upnp_configid = 1337;
 
+static const char *
+nts_to_str(int nts)
+{
+	switch(nts)
+	{
+	case NTS_SSDP_ALIVE:
+		return "ssdp:alive";
+	case NTS_SSDP_BYEBYE:
+		return "ssdp:byebye";
+	case NTS_SSDP_UPDATE:
+		return "ssdp:update";
+	}
+	return "unknown";
+}
+
 /* updateDevice() :
  * adds or updates the device to the list.
  * return value :
- *  -1 : error
- *   0 : the device was updated
+ *   0 : the device was updated (or nothing done)
  *   1 : the device was new    */
 static int
 updateDevice(const struct header * headers, time_t t)
@@ -87,7 +102,7 @@ updateDevice(const struct header * headers, time_t t)
 		  && p->headers[HEADER_USN].l == headers[HEADER_USN].l
 		  && (0==memcmp(p->headers[HEADER_USN].p, headers[HEADER_USN].p, headers[HEADER_USN].l)) )
 		{
-			//printf("found! %d\n", (int)(t - p->t));
+			/*printf("found! %d\n", (int)(t - p->t));*/
 			syslog(LOG_DEBUG, "device updated : %.*s", headers[HEADER_USN].l, headers[HEADER_USN].p);
 			p->t = t;
 			/* update Location ! */
@@ -96,7 +111,10 @@ updateDevice(const struct header * headers, time_t t)
 				p = realloc(p, sizeof(struct device)
 		           + headers[0].l+headers[1].l+headers[2].l );
 				if(!p)	/* allocation error */
+				{
+					syslog(LOG_ERR, "updateDevice() : memory allocation error");
 					return 0;
+				}
 				*pp = p;
 			}
 			memcpy(p->data + p->headers[0].l + p->headers[1].l,
@@ -183,16 +201,16 @@ SendSSDPMSEARCHResponse(int s, const struct sockaddr * sockname,
 	l = snprintf(buf, sizeof(buf), "HTTP/1.1 200 OK\r\n"
 		"CACHE-CONTROL: max-age=120\r\n"
 		/*"DATE: ...\r\n"*/
-	"ST: %s\r\n"
-	"USN: %s\r\n"
-	"EXT:\r\n"
-	"SERVER: %s\r\n"
-	"LOCATION: %s\r\n"
-		"OPT: \"http://schemas.upnp.org/upnp/1/0/\";\r\n" /* UDA v1.1 */
+		"ST: %s\r\n"
+		"USN: %s\r\n"
+		"EXT:\r\n"
+		"SERVER: %s\r\n"
+		"LOCATION: %s\r\n"
+		"OPT: \"http://schemas.upnp.org/upnp/1/0/\"; ns=01\r\n" /* UDA v1.1 */
 		"01-NLS: %u\r\n" /* same as BOOTID. UDA v1.1 */
 		"BOOTID.UPNP.ORG: %u\r\n" /* UDA v1.1 */
 		"CONFIGID.UPNP.ORG: %u\r\n" /* UDA v1.1 */
-	"\r\n",
+		"\r\n",
 		st, usn,
 		server, location,
 		upnp_bootid, upnp_bootid, upnp_configid);
@@ -298,7 +316,7 @@ containsForbiddenChars(const unsigned char * p, int len)
 #define METHOD_NOTIFY 2
 
 /* ParseSSDPPacket() :
- * parse a received SSDP Packet and call 
+ * parse a received SSDP Packet and call
  * updateDevice() or removeDevice() as needed
  * return value :
  *    -1 : a device was removed
@@ -324,7 +342,7 @@ ParseSSDPPacket(int s, const char * p, ssize_t n,
 	for(methodlen = 0;
 	    methodlen < n && (isalpha(p[methodlen]) || p[methodlen]=='-');
 		methodlen++);
-	if(methodlen==8 && 0==memcmp(p, "M-SEARCH", 8)) 
+	if(methodlen==8 && 0==memcmp(p, "M-SEARCH", 8))
 		method = METHOD_MSEARCH;
 	else if(methodlen==6 && 0==memcmp(p, "NOTIFY", 6))
 		method = METHOD_NOTIFY;
@@ -334,7 +352,7 @@ ParseSSDPPacket(int s, const char * p, ssize_t n,
 		lineend = linestart;
 		while(lineend < p + n && *lineend != '\n' && *lineend != '\r')
 			lineend++;
-		//printf("line: '%.*s'\n", lineend - linestart, linestart);
+		/*printf("line: '%.*s'\n", lineend - linestart, linestart);*/
 		/* detect name end : ':' character */
 		nameend = linestart;
 		while(nameend < lineend && *nameend != ':')
@@ -371,6 +389,8 @@ ParseSSDPPacket(int s, const char * p, ssize_t n,
 					nts = NTS_SSDP_ALIVE;
 				else if(m==11 && 0==strncasecmp(valuestart, "ssdp:byebye", 11))
 					nts = NTS_SSDP_BYEBYE;
+				else if(m==11 && 0==strncasecmp(valuestart, "ssdp:update", 11))
+					nts = NTS_SSDP_UPDATE;
 			}
 			else if(l==8 && 0==strncasecmp(linestart, "location", 8))
 				i = HEADER_LOCATION;
@@ -432,11 +452,14 @@ ParseSSDPPacket(int s, const char * p, ssize_t n,
 			printf("%d-'%.*s'\n", i, headers[i].l, headers[i].p);
 	}
 #endif
-	syslog(LOG_DEBUG,"SSDP request: '%.*s' (%d) st=%.*s",
-	       methodlen, p, method, st_len, st);
+	syslog(LOG_DEBUG,"SSDP request: '%.*s' (%d) %s %s=%.*s",
+	       methodlen, p, method, nts_to_str(nts),
+	       (method==METHOD_NOTIFY)?"nt":"st",
+	       (method==METHOD_NOTIFY)?headers[HEADER_NT].l:st_len,
+	       (method==METHOD_NOTIFY)?headers[HEADER_NT].p:st);
 	switch(method) {
 	case METHOD_NOTIFY:
-		if(headers[0].p && headers[1].p && headers[2].p) {
+		if(headers[HEADER_NT].p && headers[HEADER_USN].p && headers[HEADER_LOCATION].p) {
 			if(nts==NTS_SSDP_ALIVE) {
 				r = updateDevice(headers, time(NULL) + lifetime);
 			}
@@ -519,6 +542,8 @@ void processRequest(struct reqelem * req)
 
 	n = read(req->socket, buf, sizeof(buf));
 	if(n<0) {
+		if(errno == EINTR || errno == EAGAIN || errno == EWOULDBLOCK)
+			return;	/* try again later */
 		syslog(LOG_ERR, "(s=%d) processRequest(): read(): %m", req->socket);
 		goto error;
 	}
@@ -550,12 +575,13 @@ void processRequest(struct reqelem * req)
 			} else {
 				/* test if we can put more responses in the buffer */
 				if(d->headers[HEADER_LOCATION].l + d->headers[HEADER_NT].l
-				  + d->headers[HEADER_USN].l + 6 + (rp - rbuf) >= sizeof(rbuf))
-				  	break;
+				  + d->headers[HEADER_USN].l + 6
+				  + (rp - rbuf) >= (int)sizeof(rbuf))
+					break;
 				if( (type==1 && 0==memcmp(d->headers[HEADER_NT].p, p, l))
 				  ||(type==2 && 0==memcmp(d->headers[HEADER_USN].p, p, l))
 				  ||(type==3) ) {
-				  	/* response :
+					/* response :
 					 * 1 - Location
 					 * 2 - NT (device/service type)
 					 * 3 - usn */
@@ -736,14 +762,21 @@ error:
 	return;
 }
 
-static volatile int quitting = 0;
+static volatile sig_atomic_t quitting = 0;
 /* SIGTERM signal handler */
 static void
 sigterm(int sig)
 {
-	signal(sig, SIG_IGN);
+	(void)sig;
+	/*int save_errno = errno;*/
+	/*signal(sig, SIG_IGN);*/
+#if 0
+	/* calling syslog() is forbidden in a signal handler according to
+	 * signal(3) */
 	syslog(LOG_NOTICE, "received signal %d, good-bye", sig);
+#endif
 	quitting = 1;
+	/*errno = save_errno;*/
 }
 
 /* main(): program entry point */
@@ -825,7 +858,7 @@ int main(int argc, char * * argv)
 			LOG_MINISSDPD);
 	if(!debug_flag) /* speed things up and ignore LOG_INFO and LOG_DEBUG */
 		setlogmask(LOG_UPTO(LOG_NOTICE));
-	
+
 	if(checkforrunning(pidfilename) < 0)
 	{
 		syslog(LOG_ERR, "MiniSSDPd is already running. EXITING");
@@ -950,8 +983,11 @@ int main(int argc, char * * argv)
 		if(select(FD_SETSIZE, &readfds, 0, 0, 0) < 0)
 		{
 			if(errno != EINTR)
+			{
 				syslog(LOG_ERR, "select: %m");
-			break;
+				break;	/* quit */
+			}
+			continue;	/* try again */
 		}
 #ifdef ENABLE_IPV6
 		if((s_ssdp6 >= 0) && FD_ISSET(s_ssdp6, &readfds))
@@ -1042,6 +1078,8 @@ int main(int argc, char * * argv)
 			else
 			{
 				syslog(LOG_INFO, "(s=%d) new request connection", s);
+				if(!set_non_blocking(s))
+					syslog(LOG_WARNING, "Failed to set new socket non blocking : %m");
 				tmp = malloc(sizeof(struct reqelem));
 				if(!tmp)
 				{
