@@ -1,7 +1,7 @@
-/* $Id: openssdpsocket.c,v 1.8 2011/10/07 09:21:03 nanard Exp $ */
+/* $Id: openssdpsocket.c,v 1.12 2012/05/21 17:13:11 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
- * (c) 2006-2011 Thomas Bernard
+ * (c) 2006-2012 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
@@ -17,6 +17,7 @@
 #include <syslog.h>
 
 #include "openssdpsocket.h"
+#include "upnputils.h"
 
 /* SSDP ip/port */
 #define SSDP_PORT (1900)
@@ -36,7 +37,7 @@ GetIfAddrIPv4(const char * ifaddr)
 	int s;
 	struct ifreq ifr;
 	int ifrlen;
-	
+
 	/* let's suppose ifaddr is a IPv4 address
 	 * such as 192.168.1.1 */
 	addr = inet_addr(ifaddr);
@@ -44,7 +45,7 @@ GetIfAddrIPv4(const char * ifaddr)
 	{
 		return addr;
 	}
-	/* let's suppose the ifaddr was in fact an interface name 
+	/* let's suppose the ifaddr was in fact an interface name
 	 * such as eth0 */
 	s = socket(PF_INET, SOCK_DGRAM, 0);
 	if(s < 0)
@@ -82,6 +83,8 @@ AddDropMulticastMembership(int s, const char * ifaddr, int ipv6, int drop)
 	unsigned int ifindex;
 #endif
 
+	if(s <= 0)
+		return -1;	/* nothing to do */
 #ifdef ENABLE_IPV6
 	if(ipv6)
 	{
@@ -92,8 +95,9 @@ AddDropMulticastMembership(int s, const char * ifaddr, int ipv6, int drop)
 		if(setsockopt(s, IPPROTO_IPV6, drop ? IPV6_LEAVE_GROUP : IPV6_JOIN_GROUP,
 		   &mr, sizeof(struct ipv6_mreq)) < 0)
 		{
-			syslog(LOG_ERR, "setsockopt(udp, %s)(%s): %m",
+			syslog(LOG_ERR, "setsockopt(udp, %s)(%s, %s): %m",
 			       drop ? "IPV6_LEAVE_GROUP" : "IPV6_JOIN_GROUP",
+			       LL_SSDP_MCAST_ADDR,
 			       ifaddr);
 			return -1;
 		}
@@ -101,8 +105,9 @@ AddDropMulticastMembership(int s, const char * ifaddr, int ipv6, int drop)
 		if(setsockopt(s, IPPROTO_IPV6, drop ? IPV6_LEAVE_GROUP : IPV6_JOIN_GROUP,
 		   &mr, sizeof(struct ipv6_mreq)) < 0)
 		{
-			syslog(LOG_ERR, "setsockopt(udp, %s)(%s): %m",
+			syslog(LOG_ERR, "setsockopt(udp, %s)(%s, %s): %m",
 			       drop ? "IPV6_LEAVE_GROUP" : "IPV6_JOIN_GROUP",
+			       SL_SSDP_MCAST_ADDR,
 			       ifaddr);
 			return -1;
 		}
@@ -121,7 +126,7 @@ AddDropMulticastMembership(int s, const char * ifaddr, int ipv6, int drop)
 			       ifaddr);
 			return -1;
 		}
-		
+
 		if (setsockopt(s, IPPROTO_IP, drop ? IP_DROP_MEMBERSHIP : IP_ADD_MEMBERSHIP,
 		    (void *)&imr, sizeof(struct ip_mreq)) < 0)
 		{
@@ -150,7 +155,7 @@ OpenAndConfSSDPReceiveSocket(int n_listen_addr,
 	struct sockaddr_in sockname;
 #endif
 	socklen_t sockname_len;
-	
+
 #ifdef ENABLE_IPV6
 	if( (s = socket(ipv6 ? PF_INET6 : PF_INET, SOCK_DGRAM, 0)) < 0)
 #else
@@ -159,8 +164,12 @@ OpenAndConfSSDPReceiveSocket(int n_listen_addr,
 	{
 		syslog(LOG_ERR, "socket(udp): %m");
 		return -1;
-	}	
-	
+	}
+
+	if(!set_non_blocking(s)) {
+		syslog(LOG_WARNING, "Failed to set SSDP socket non blocking : %m");
+	}
+
 #ifdef ENABLE_IPV6
 	memset(&sockname, 0, sizeof(struct sockaddr_storage));
 	if(ipv6)
