@@ -1,11 +1,11 @@
-/* $Id: openssdpsocket.h,v 1.4 2011/07/29 15:21:13 nanard Exp $ */
+/* $Id: openssdpsocket.h,v 1.5 2012/09/27 15:40:29 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
  * (c) 2006-2011 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
-#ifndef __OPENSSDPSOCKET_H__
-#define __OPENSSDPSOCKET_H__
+#ifndef OPENSSDPSOCKET_H_INCLUDED
+#define OPENSSDPSOCKET_H_INCLUDED
 
 /**
  * Open a socket and configure it for receiving SSDP packets
